@@ -18,6 +18,8 @@
 #include <bonobo/bonobo-ui-node-private.h>
 #include <bonobo/bonobo-ui-toolbar-control-item.h>
 
+G_BEGIN_DECLS
+
 /* To debug plug/socket/control render issues */
 #define noDEBUG_CONTROL
 
@@ -35,6 +37,10 @@ void       bonobo_ui_toolbar_control_item_set_sensitive
                                           (BonoboUIToolbarControlItem *item,
 					   gboolean                    sensitive);
 
+
+gboolean   bonobo_widget_clobber_focus    (GtkWidget        *widget,
+					   GtkDirectionType  direction);
+
 #ifndef   DEBUG_UI
 
 static inline void dprintf (const char *format, ...) { };
@@ -47,5 +53,7 @@ static inline void dprintf (const char *format, ...) { };
 
 #endif /* DEBUG_UI */
 
+G_END_DECLS
+
 #endif /* _BONOBO_UI_PRIVATE_H_ */
 
